#==============calibration_qc==============

import math
from ExceptionHandling import ExceptionHandling

class CallibrationQC:
    def __init__(self,**kwargs):
        pass

class CalibrationQCVac(CallibrationQC):
    
    def __init__(self,**kwargs):
        super().__init__(**kwargs)
        self.vacuum_values=kwargs["vacuum_values"]
    
    def calculate_calibration_coefficients(self):
        a=(self.vacuum_values["off_state_vacuum"] - self.vacuum_values["on_state_vacuum"])/( self.vacuum_values["off_state_observed_vacuum"] - self.vacuum_values["on_state_observed_vacuum"]) 
        b = self.vacuum_values["off_state_vacuum"] - a*self.vacuum_values["off_state_observed_vacuum"]
        return a,b
    
    def check_exceptions(self):
        new_exception_handling=ExceptionHandling()
        return new_exception_handling.Numeric_Input_Type_Checking(self.vacuum_values.values())

    def calibrate(self):   
        if self.check_exceptions()!=0:
            return {"insufficient_data_indicator":1}
        a,b =self.calculate_calibration_coefficients()
        return {"calibration_parameters" : {"a": a, "b": b}, "insufficient_data_indicator": 0}
            
        
class CalibrationQCPF(CallibrationQC):
    
    def __init__(self,**kwargs):
        super().__init__(**kwargs)
        self.voltage_values=kwargs["voltage_values"]
        self.current_values=kwargs["current_values"]
        self.previous_calibration_parameters=kwargs["previous_calibration_parameters"]
    
    def calculate_calibration_coefficients(self):
        self.a_voltage = ( self.voltage_values["off_state_voltage"] - self.voltage_values["on_state_voltage"] )/( self.voltage_values["modified_observed_off_state_voltage"] - self.voltage_values["modified_observed_on_state_voltage"] )
        self.a_current = ( self.current_values["off_state_current"] - self.current_values["on_state_current"] )/( self.current_values["modified_observed_off_state_current"] - self.current_values["modified_observed_on_state_current"] )
        self.b_voltage=self.voltage_values["off_state_voltage"]- self.a_voltage*self.voltage_values["modified_observed_off_state_voltage"]
        self.b_current = self.current_values["off_state_current"] - self.a_current*self.current_values["modified_observed_off_state_current"]
        if sum(self.previous_calibration_parameters["previous_voltage_b"]) != 0:
            self.b_voltage = self.previous_calibration_parameters["previous_voltage_b"]
        if sum( self.previous_calibration_parameters["previous_current_b"] ) != 0:
            self.b_current = self.previous_calibration_parameters["previous_current_b"]

    def generate_pf_calibration_params(self):
        return {"calibration_parameters": {"voltage_phase1_a": {"v1":round(self.a_voltage[0],3)}, "current_phase1_a": {"i1":round(self.a_current[0],3)},
                                            "voltage_phase2_a" : {"v2":round(self.a_voltage[1],3)}, "current_phase2_a": {"i2":round(self.a_current[1],3)},
                                            "voltage_phase3_a" : {"v3":round(self.a_voltage[2],3)}, "current_phase3_a": {"i3":round(self.a_current[2],3)},
                                            "voltage_phase1_b" : {"v1":round(self.b_voltage[0],3)}, "current_phase1_b": {"i1":round(self.b_current[0],3)},
                                            "voltage_phase2_b" : {"v2":round(self.b_voltage[1],3)}, "current_phase2_b": {"i2":round(self.b_current[1],3)},
                                            "voltage_phase3_b" : {"v3":round(self.b_voltage[2],3)}, "current_phase3_b": {"i3":round(self.b_current[2],3)}},
                    "insufficient_data_indicator": 0}
        
    def calibrate(self):
        self.calculate_calibration_coefficients()
        if sum(math.isfinite(x) for x in self.a_current)==3 and sum(math.isfinite(x) for x in self.a_voltage)!=3:
            return {"insufficient_data_indicator": 1}
        return self.generate_pf_calibration_params()