import json
import warnings,sys,traceback
from collections import namedtuple
import getopt
import sys
from SensorQC import SensorQCVac, SensorQCPF
from CallibrationQC import CalibrationQCVac, CalibrationQCPF
from AdditionalFunctions import AdditionalFunctions
import os

warnings.filterwarnings("ignore")
config_file = None
json_string,input_json="",{}
dconfig={
    "base_path": "",
    "qc_info_api": "https://api.machinesense.com/api/sensor/"
}
debug_enable=False


def eprint(*args,**kwargs):
    if debug_enable:
        print(*args,**kwargs)
    else:
        print(*args,file=sys.stderr,**kwargs)

def read_config(config_file):
    temp_config=dconfig
    if config_file is not None:
        with open(config_file) as f:
            temp_config = json.load(f)
            f.close()
    return temp_config

def parse_json_obj(json0):
    json_obj = json.loads(json0, object_hook=lambda d: namedtuple("X", d.keys())(*d.values()))
    json_string=json.loads(json0)
    input_json=json_obj[0]
    return json_string, input_json

def get_sensor_indicator(string, sensor_id):
    return 1 if string in sensor_id.lower() else 0

def get_vac_qc_output(sensor_pass_fail_reason_list,sensor_qc_vac_called,vacuum_values,sensor_calibration_vac_called):
    if sensor_qc_vac_called["insufficient_data_indicator"] != 0 :
        return {"calibration_parameters": [], "reason": "Insufficient Data for Sensor QC", "success": "false"}
    if sensor_qc_vac_called["result"] != "Passed":      
        return {"calibration_parameters":[],"reason": sensor_pass_fail_reason_list, "success" :"false"}
    if vacuum_values["off_state_observed_vacuum"] - vacuum_values["on_state_observed_vacuum"] == 0:
        return {"calibration_parameters" : [], "reason" :"ON state and OFF state observed vac values are same : "+ str(vacuum_values["off_state_observed_vacuum"]) ,"success": "false" }
    if sensor_calibration_vac_called["insufficient_data_indicator"] != 0:    
       return {"calibration_parameters":[],"reason" : "Sensor QC has Passed and Insufficent Data for sensor calibration", "success" : "false"}
    if sensor_calibration_vac_called["calibration_parameters"]["a"] == 0:
        return {"calibration_parameters":[],"reason" : "Calibration failed as a = 0", "success" : "false"}
    return {"calibration_parameters":sensor_calibration_vac_called["calibration_parameters"],"reason":sensor_pass_fail_reason_list, "success":"true"}

def perform_vac_qc(vac_sensor_indicator,ps_sensor_indicator):
    sensorqc = SensorQCVac(base_path=configs["base_path"],input_json=input_json,vac_sensor_indicator=vac_sensor_indicator,ps_sensor_indicator=ps_sensor_indicator)
    sensor_qc_vac_called = sensorqc.process_qc()
    #print(sensor_qc_vac_called)
    if sensor_qc_vac_called['insufficient_data_indicator'] != 0:
        return get_vac_qc_output([],sensor_qc_vac_called,{},{})
    sensor_pass_fail_reason_list = sensor_qc_vac_called["sensor_pass_fail_reason_list"]
    vacuum_values = sensor_qc_vac_called["vacuum_values"]
    calibration= CalibrationQCVac(vacuum_values=vacuum_values)
    sensor_calibration_vac_called = calibration.calibrate()
    return get_vac_qc_output(sensor_pass_fail_reason_list,sensor_qc_vac_called,vacuum_values,sensor_calibration_vac_called)        

def get_pf_qc_output(parameters_pf_called,sensor_qc_pf_called,sensor_pass_fail_reason,sensor_calibration_pf_called, failure_reason):
    if parameters_pf_called["isostat_exception_indicator"] != 0 :
        return {"calibration_parameters":[], "reason":"isostat NOT found in api : " , "success" : "false"}
    if sensor_qc_pf_called["insufficient_data_indicator"] != 0:
        return {"calibration_parameters": [],"reason":"Insufficient Data for Sensor QC","success":"false"}
    if sensor_qc_pf_called["result"] != "Passed" : 
        return {"calibration_parameters":[],"reason":"Sensor QC has Failed",
        "sensor_pass_fail_reason": sensor_pass_fail_reason,"failure_reason":failure_reason,"success":"false"}
    if sensor_calibration_pf_called["insufficient_data_indicator"] != 0:         
        return  {"calibration_parameters": [],"reason":"Sensor QC has Passed and Insufficent Data for sensor calibration", "success": "false"}    
    return  {"calibration_parameters": sensor_calibration_pf_called["calibration_parameters"],"sensor_pass_fail_reason":sensor_pass_fail_reason,"reason":"Sensor QC has Passed","success":"true"}

def get_qc_information_old():
    new_additional_functions=AdditionalFunctions(configs["qc_info_api"])
    qc_information_api_called = new_additional_functions.QC_Information_API_Formation( json_string ) 
    qc_information_api_data_pf = qc_information_api_called["qc_information_api_data_pf"] 
    qc_information_api = qc_information_api_called["qc_information_api"]
    return qc_information_api_data_pf,qc_information_api,new_additional_functions.Get_Parameters_PF(qc_information_api_data_pf, qc_information_api)

#"cttype": "100A", "loadtype": "Normal", "isostat": "Yes"

def get_pf_sensor_config(sensor_id):
   qc_information_api_data = get_sensor_conf_file(sensor_id)
   print("qc_information_api_data: ",qc_information_api_data)
   return qc_information_api_data
   #forming_api = "http://localhost"
   #return {"qc_information_api_data_pf" : qc_information_api_data, "qc_information_api": forming_api}

def get_sensor_conf_file(sensor_id):
    x = sensor_id.split("_")
    file_id = x[2]
    conf_dir="/opt/novatec/data/slave/confs"
    file = os.path.join(conf_dir,f"prophecy_hw_{file_id}.conf")
    with open(file) as f:
        j = json.load(f)
    return j

def transform_pf_api(pf_api):
    return {
        "cttype": float(pf_api['cttype'].replace("A","")),
        "isostat": "Yes",
        "cttype_exception_indicator": 0,
        "isostat_exception_indicator": 0,
        "final_cttype_api": "http://localhost"
    }

def get_qc_information():
    latest_attempt_no=len(json_string["calibrationAttempts"])
    pf_sensor_id=json_string["calibrationAttempts"][latest_attempt_no-1]["sensorId"]


    #new_additional_functions=AdditionalFunctions(configs["qc_info_api"])
    #qc_information_api_called = new_additional_functions.QC_Information_API_Formation( json_string ) 
    #qc_information_api_called = get_pf_sensor_config(pf_sensor_id) 
    #qc_information_api_data_pf = qc_information_api_called["qc_information_api_data_pf"] 
    qc_information_api_data_pf = get_pf_sensor_config(pf_sensor_id) 
    #qc_information_api_data_pf = qc_information_api_called["qc_information_api_data_pf"] 
    #qc_information_api = qc_information_api_called["qc_information_api"]
    #return qc_information_api_data_pf,qc_information_api,new_additional_functions.Get_Parameters_PF(qc_information_api_data_pf, qc_information_api)
    qc_information_api = None
    return qc_information_api_data_pf,transform_pf_api(qc_information_api_data_pf)


def perform_pf_qc():
    qc_information_api_data_pf,parameters_pf_called=get_qc_information()
    print("qc_information_api_data_pf: ",json.dumps(qc_information_api_data_pf,indent=4))
    print("parameters_pf_called: ",json.dumps(parameters_pf_called,indent=4))

    sensorqc=SensorQCPF(base_path=configs["base_path"],input_json=input_json,qc_information_api_data_pf=qc_information_api_data_pf,parameters_pf_called=parameters_pf_called)
    sensor_qc_pf_called = sensorqc.process_qc()
    #print(sensor_qc_pf_called)
    failure_reason=""
    if sensor_qc_pf_called['insufficient_data_indicator'] != 0:
        return get_pf_qc_output(parameters_pf_called,sensor_qc_pf_called,qc_information_api,"",[],failure_reason)
    previous_calibration_parameters = sensor_qc_pf_called['previous_calibration_parameters']
    if sensor_qc_pf_called['result']!= 'Passed' :
        failure_reason = sensor_qc_pf_called['failure_reason'] 
    sensor_pass_fail_reason = sensor_qc_pf_called['sensor_pass_fail_reason_list']
    voltage_values = sensor_qc_pf_called["voltage_values"]
    current_values = sensor_qc_pf_called["current_values"]
    new_callibration_QC=CalibrationQCPF(voltage_values=voltage_values, current_values=current_values, previous_calibration_parameters=previous_calibration_parameters)
    sensor_calibration_pf_called = new_callibration_QC.calibrate()
    return get_pf_qc_output(parameters_pf_called,sensor_qc_pf_called,sensor_pass_fail_reason,sensor_calibration_pf_called,failure_reason)

def main_output():
    try:
        if not input_json:
            return "Calibration List is blank"   
        sensor_id=input_json[0].sensorId;lsensor_id=sensor_id.lower()
        vac_sensor_indicator= get_sensor_indicator("vac",lsensor_id)
        ps_sensor_indicator= get_sensor_indicator("ps",lsensor_id)
        pf_sensor_indicator= get_sensor_indicator("pf",lsensor_id)
        if 1 not in {vac_sensor_indicator,ps_sensor_indicator}:
            if pf_sensor_indicator != 1:
                return None
            return perform_pf_qc()
        return perform_vac_qc(vac_sensor_indicator,ps_sensor_indicator)
    except Exception as  e:
        eprint("unexpected error", traceback.format_exc())
        return list([e])


def run_calibration(json_profile=None):
    global config_file
    global json_string
    global input_json
    global configs

    configs=read_config(config_file)
    #json_path = args[0]
    json_path = json_profile
    json0= open(json_path).read()
    json_string,input_json=parse_json_obj(json0)
    output = main_output()
    return output

if __name__ == '__main__':
    try:
        opts, args = getopt.getopt(sys.argv[1:],"c:d:")
    except getopt.GetoptError:
        eprint("Error: Error Loading Configuration Details: ")
    for opt,arg in opts:
        if opt == "-c":
            config_file = arg.strip()
        elif opt == "-d":
            debug_enable=True
    output = run_calibration(json_profile=args[0])
    print(json.dumps(output))
    #configs=read_config(config_file)
    #json_path = args[0]
    #json0= open(json_path).read()
    #json_string,input_json=parse_json_obj(json0)
    #output = main_output()
    #print(json.dumps(output))        
