
from VacSensorCollector import SensorCollector, get_datetime
from zmq_convert import get_zmq_data_format2
from pynovatec.events import EventPost
from qc_events import QCStateChange,SensorDataPoint,SensorDataCollectionComplete,SensorDataBatchCollectionComplete,SensorDataBatchCollectionStarted,SensorDataCollectionError


class PFSensorCollector(SensorCollector):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        #self.sensor_id=f"{self.mac}_{self.package_type}_{self.package_index}"
        self.collect_points = 60
        self.off_threshold = 10
        self.on_threshold = 200
        if not self.contents:
            self.contents = {
               "on": "273.5,34.7,271,34.7,271,34.7",
               "off": "0,0,0,0,0,0"
            }
        self.set_content(self.contents['off'])

    def process_data_stream(self,data):
        print(f"device_state({self.device_state}) -- expected_state({self.expected_state})")
        if self.device_state == self.expected_state:
            self.start_time=int(data['timeStamp']) if not self.save_list else self.start_time
            self.last_time_seen=int(data['timeStamp'])
            data_set=','.join([ f'{float(x)}' for x in data['values']])
            self.add_data([get_datetime(int(data['timeStamp'])),int(data['index']),int(data['dataType']),"",data_set])

    # Stubbing these for now so I can trick ZMQ Collector so it doesn't reset these hard-coded values as if it was a 
    # pump until I fix this.

    def set_on_threshold(self,value):
        pass
                                                                                                     
    def set_off_threshold(self,value):
        pass

    def set_state(self,data):
        value=float(data['values'][0])                                               
        print("Value: ",value)
        print("off threshold: ",self.off_threshold)
        if value >= self.on_threshold + 0.5:
            self.update_device_state('on')
        elif value <= self.off_threshold - 1.0:
           self.update_device_state('off')

    def process_data(self,data):
        mac_ds=data['mac'].lower()
        print("Data: ",data)
        if data['sensorType'].upper()!=self.package_type or mac_ds!=self.mac:
            print("No match 1")
            print("mac_ds: ",mac_ds)
            print("self.mac.lower(): ", self.mac.lower())
            return
        if data['dataType'] != '4':
            print("No match 2")
            return
        if int(data['index']) != int(self.package_index):
            print("no match 3")
            return
        print("Device: ",mac_ds,"Queue Length: ",len(self.save_list)) 
        self.set_state(data)
        self.process_data_stream(data)
        EventPost(SensorDataPoint(data))

    def format_data_stream(self,line):
        data=get_zmq_data_format2(line)
        return self.process_data(data)

