import zmq
from pynovatec.events import EventListener,EventRegisterListener,EventPost,EventUnregisterListener
from qc_events import QCStateChange,SensorDataPoint,QCEvents,SensorDataCollectionComplete, SensorDataCalibrationComplete
from qc_events import QCListener
import threading
from zmq_convert import get_zmq_data_format2

class ZmqCollection(EventListener,threading.Thread):
#class ZmqCollection(threading.Thread):
    def __init__(self,collector,*args,collectors=None,host="127.0.0.1",port=7778,**kwargs):
        self.host=host
        self.collector=collector
        if collectors:
            self.collectors = collectors
        else:
            self.collectors = [collector]
        self.port = port
        #super(EventListener,self).__init__(*args,**kwargs)
        EventListener.__init__(self,*args,**kwargs)
        threading.Thread.__init__(self,target=self.run)
        #super().__init__(target=self.run)
        self.events.append(QCEvents)
        EventRegisterListener(self)
        self.mapCollectors()
        self.keepRunning = True
        for collector in self.collectors:
            collector.clear_profile()

    def mapCollectors(self):
        self.collectorMap={}
        for collector in self.collectors:
            self.collectorMap[collector.sensor_id] = {}
            self.collectorMap[collector.sensor_id]['collector']=collector
            self.collectorMap[collector.sensor_id]['complete']=False

    def allComplete(self):
        for key,cm in self.collectorMap.items():
            print("CM: ",cm)
            if not cm['complete']:
                return False
        return True

    def handle(self,msg):
        #print(f"ZMQ Event Handle ({threading.current_thread().ident}): ",msg.data)
        print(f"ZMQ Event Handle ({self.ident}): ",msg.data)
        if msg.data['event_name'] == "batch_collection_complete":
            collector = self.collectorMap[msg.data['data']['sensor_id']]['collector']
            print("Data point collection complete.")
            profile_file = collector.save(msg.data['data']['endtime'])
            #if self.collector.device_state == "off":
            if msg.data['data']['collected_state'] == "off":
                collector.set_on_collection()
                collector.enable_collection()
            elif msg.data['data']['collected_state'] == "on":
                EventPost(SensorDataCollectionComplete({
                    "profile_file": profile_file,
                    "sensor_id": collector.sensor_id
                }))
                self.collectorMap[collector.sensor_id]['complete'] = True
                if self.allComplete():
                    EventPost(SensorDataCalibrationComplete({
                        'msg': "Calibration completed."
                    }))
                    self.keepRunning=False

    def run(self):
        self.zmq_stream_data()

    def stop(self):
        self.keepRunning = False

    def handle_message(self,string):
        data=get_zmq_data_format2(string.decode())
        for collector in self.collectors:
            #collector.format_data_stream(string.decode())
            collector.process_data(data)

    def zmq_stream_data(self):
        context = zmq.Context()
        socket = context.socket(zmq.SUB)

        print("Collecting data from port %s..." % self.port)
        socket.connect ("tcp://127.0.0.1:%s" % self.port)
        socket.subscribe("")
        for collector in self.collectors:
            collector.enable_collection()
        while self.keepRunning:
            string = socket.recv()
            self.handle_message(string)
            #print(f"RECV: {string}")
            #select_data_stream(string.decode())

        print("Shutting down zmq stream...")
        socket.close()
        context.term()
        EventUnregisterListener(self)

